log_prv:db 0EA
        dd ?
log_tag:db 'LOG daemon',0


;
; log_isr
;

log_isr:nop
        ; to little services within this daemon to use a look-up table
        cmp ah,1
        je log_write
        cmp ah,0
        if e jmp log_open
        cmp ah,2
        if e jmp log_close

        ; return error
        mov ah,-1
        iret

;
; log_write
;
;    BL = priority
; DS:SI = pointer to message (0 terminated)
; ES:DI = pointer to log_task struct
;

log_write:
        push bx,cx,dx,si,di,bp

        ; if socket is cleared then open udp socket
        es cmp [di]socket w,-1
        jnz> L1

        ; open udp socket
        call log_open_udp
        es cmp [di]socket w,-1
        jnz> L1

        ; return error
        mov ah,1
        jmp> L2

L1:     ; point to buffer
        mov bp,di
        add di,buffer

        ; calculate fac.pri code
        es mov bh,[bp]facility b
        shl bh,3
        add bl,bh
        xor bh,bh

        push si,ds

        ; fetch ident pointer
        es mov ds,[bp]ident_ptr.sg w
        es mov si,[bp]ident_ptr.off w

        ; store fac.pci
        mov al,'<'
        stosb
        call bin2dec
        mov al,'>'
        stosb

        ; store ident
L1:     lodsb
        or al,al
        jz> L2
        stosb
        jmp L1

L2:     pop ds,si

        ; need to store pid?
        es test [bp]option b,LOG_PID
        jz> L1

        ; store pid
        mov al,'['
        stosb
        es mov bx,[bp]task_id w
        call bin2dec
        mov al,']'
        stosb

L1:     ; store border
        mov ax,' ' BY ':'
        stosw

        ; store message
L1:     lodsb
        stosb
        or al,al
        jnz L1

        ; calculate message length
        mov cx,di
        sub cx,bp
        sub cx,buffer

        ; update the send_params
        es mov [bp]sparams.buffer_length w,cx

        ; write the message
        mov ah,6
        mov bx,[bp]socket w
        mov dx,es
        mov si,bp
        add si,sparams
        int 0AC

        ; need to log to console to?
        es test [bp]option b,LOG_PERROR
        jz> L1

        ; terminate the message using a '$'
        dec di
        mov ax,0a BY 0d
        stosw
        mov al,'$'
        stosb

        ; remove leading '<%d>'
        mov di,bp
        add di,buffer
        mov al,'>'
        repne scasb

        ; log message to console
        push ds
        mov ah,9
        mov ds,es
        mov dx,di
        int 021
        pop ds

L1:     ; return success
        xor ah,ah

L2:     pop bp,di,si,dx,cx,bx
        iret

;
; log_open_udp
;
; ES:DI = pointer to log_task struct
;

log_open_udp:
        ; open an udp socket
        mov ah,1
        mov al,SOCK_DGRAM
        int 0AC

        ; check for error
        or dx,dx
        jz> L1

        ; if LOG_CONS then report error
        es test [di]option b,LOG_CONS
        if z ret

        ; report error
        ; ...

        ret

L1:     ; store socket
        es mov [di]socket w,ax
        ret

;
; log_open
;
;    BH = facility
;    BL = option
; DS:SI = pointer to ident (0 terminated)
; ES:DI = pointer to log_task struct
;

log_open:
        push dx

        ; store registers in log_task
        es mov [di]facility b,bh
        es mov [di]option b,bl

        ; store pointer to ident
        es mov [di]ident_ptr.sg w,ds
        es mov [di]ident_ptr.off w,si

        ; find out which task is calling us
        mov ah,4
        int 0AD
        es mov [di]task_id w,ax

        ; clear socket
        es mov [di]socket w,-1

        ; fill in some of the send_params files
        es mov [di]sockaddr.sin_family w,AF_INET

        ; store log host
        cs mov ax,log_host[2]w
        cs mov dx,log_host[0]w
        es mov [di]sockaddr.sin_addr.s_addr[2]w,ax
        es mov [di]sockaddr.sin_addr.s_addr[0]w,dx

        ; store log port
        xmv dx,LOG_PORT
        es mov [di]sockaddr.sin_port w,dx

        ; store pointer to sockaddr
        es mov [di]sparams.to_ptr.sg w,es
        es mov [di]sparams.to_ptr.off w,di
        es add [di]sparams.to_ptr.off w,sockaddr
        es mov [di]to_length w,TYPE send_params
        es mov [di]sparams.to_length_ptr.sg w,es
        es mov [di]sparams.to_length_ptr.off w,di
        es add [di]sparams.to_length_ptr.off w,to_length

        ; store pointer to message buffer
        es mov [di]sparams.buffer_ptr.sg w,es
        es mov [di]sparams.buffer_ptr.off w,di
        es add [di]sparams.buffer_ptr.off w,buffer
        es mov [di]sparams.buffer_length w,0

        ; if not LOG_ODELAY open udp socket
        test bl,LOG_ODELAY
        if z call log_open_udp

        ; return success
        xor ah,ah

        pop dx
        iret

;
; log_close
;
; ES:DI = pointer to log_task struct
;

log_close:
        push bx

        ; close udp socket
        mov ah,2
        es mov bx,[di]socket w
        int 0AC

        ; clear socket
        es mov [di]socket w,-1

        ; return success
        xor ah,ah

        pop bx
        iret

